/*****************************************************************
*    Un GPS de randonée avec un ATMEGA328P avec affichage de la  *
*latitude et longitude ainsi que la mémorisation du point de     *
*départ... L'heure, le temps de déplacement et la direction      *
*angulaire du point de départ sont affichée, ainsi que la        *
* distance à vol d'oiseau ... L'heure est réglée en fonction de   *
*UTC pour être déplacé partout !                                 *
*****************************************************************/

#include <SoftwareSerial.h>
#include <TinyGPS.h>
#include <LiquidCrystal.h>
#include <EEPROM.h>
#include <avr/pgmspace.h>

LiquidCrystal lcd(13, 12, 11, 10, 9, 8);  //Pin LCD: RS,E, D4-D7,RW>0V,
SoftwareSerial gpsSerial(3, 2); // Pin communication série RX, TX
TinyGPS gps;

static int vref = 276; // Ajustement de le tension batterie 1023/3,7 volts 
byte batt = 17; // Pin entrée mesure de la tension batterie     
byte sw_sel = 14;// Validation par appui sur le bouton du codeur...
byte sw_up = 15;// Incrémentation menu sens montée
byte sw_dwn = 16;//  Décrémentation menu sens descente
byte fonction = 0;
int gmt; //  Décalage par rapport à heure universelle 
long Year_Min;  //Heure et datation
unsigned long time_On; //Temps de marche
unsigned long time_run; //Temps de déplacement
unsigned long time_old; //Temps stocké en EEprom si pas de RaZ

int year; 
int menu_index = 0; //Déclaration et mise à zéro du menu
int cum_n;
static float rad2deg = 0.01745;
float Odometre;
char msg[6];
byte LCD_LED_N = 220;  //Eclairage de l'écran (220/250)
byte month, day, hour, minute, second, sped;
long lat, lon;   //Latitude, Longitude et Angle
long lat2 = 0;
long lon2 = 0;
long wp_lat, wp_lon ; //Stokage en EEprom du point mémorisé

const char*azimuts[] = {"N  ", "NE ", "E  ", "SE ", "S  ",  "SW ",  "W  ", "NW " };  // Caractères pour la boussole

// Masques d'extraction des infos de trames GPS 

const static byte gpsDefault[] PROGMEM = {0xB5, 0x62, 0x06, 0x09, 0x0D, 0x00, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xFF, 0xFF, 0x00, 0x00, 0x07, 0x1F, 0x9E};
const static byte gpsSave[] PROGMEM = {0xB5, 0x62, 0x06, 0x09, 0x0D, 0x00, 0x00, 0x00, 0x00, 0x00, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x17, 0x31, 0xBF, 0x13};
const static byte gpsSleep[] = {0xB5, 0x62, 0x06, 0x04, 0x04, 0x00, 0x00, 0x00, 0x08, 0x00, 0x16, 0x74};
const static byte gpsHot[]  = {0xB5, 0x62, 0x06, 0x04, 0x04, 0x00, 0x00, 0x00, 0x02, 0x00, 0x10, 0x68};
const static byte Disable_GPDTM[] PROGMEM = {0x24, 0x45, 0x49, 0x47, 0x50, 0x51, 0x2c, 0x44, 0x54, 0x4d, 0x2a, 0x33, 0x42, 0x0d, 0x0a, 0xb5, 0x62, 0x06, 0x01, 0x03, 0x00, 0xf0, 0x0a, 0x00, 0x04, 0x23}; //->Disable_GPDTM
const static byte Disable_GPGBS[] PROGMEM = {0x24, 0x45, 0x49, 0x47, 0x50, 0x51, 0x2c, 0x47, 0x42, 0x53, 0x2a, 0x33, 0x30, 0x0d, 0x0a, 0xb5, 0x62, 0x06, 0x01, 0x03, 0x00, 0xf0, 0x09, 0x00, 0x03, 0x21}; //->Disable_GPGBS
const static byte Disable_GPGGA[] PROGMEM = {0x24, 0x45, 0x49, 0x47, 0x50, 0x51, 0x2c, 0x47, 0x47, 0x41, 0x2a, 0x32, 0x37, 0x0d, 0x0a, 0xb5, 0x62, 0x06, 0x01, 0x03, 0x00, 0xf0, 0x00, 0x00, 0xfa, 0x0f}; //>Disable_GPGGA
const static byte Disable_GPGLL[] PROGMEM = {0x24, 0x45, 0x49, 0x47, 0x50, 0x51, 0x2c, 0x47, 0x4c, 0x4c, 0x2a, 0x32, 0x31, 0x0d, 0x0a, 0xb5, 0x62, 0x06, 0x01, 0x03, 0x00, 0xf0, 0x01, 0x00, 0xfb, 0x11}; //->Disable_GPGLL
const static byte Disable_GPGRS[] PROGMEM = {0x24, 0x45, 0x49, 0x47, 0x50, 0x51, 0x2c, 0x47, 0x52, 0x53, 0x2a, 0x32, 0x30, 0x0d, 0x0a, 0xb5, 0x62, 0x06, 0x01, 0x03, 0x00, 0xf0, 0x06, 0x00, 0x00, 0x1b}; //->Disable_GPGRS
const static byte Disable_GPGSA[] PROGMEM = {0x24, 0x45, 0x49, 0x47, 0x50, 0x51, 0x2c, 0x47, 0x53, 0x41, 0x2a, 0x33, 0x33, 0x0d, 0x0a, 0xb5, 0x62, 0x06, 0x01, 0x03, 0x00, 0xf0, 0x02, 0x00, 0xfc, 0x13}; //->DisableGPGSA
const static byte Disable_GPGST[] PROGMEM = {0x24, 0x45, 0x49, 0x47, 0x50, 0x51, 0x2c, 0x47, 0x53, 0x54, 0x2a, 0x32, 0x36, 0x0d, 0x0a, 0xb5, 0x62, 0x06, 0x01, 0x03, 0x00, 0xf0, 0x07, 0x00, 0x01, 0x1d}; //->Disable_GPGST
const static byte Disable_GPGSV[] PROGMEM = {0x24, 0x45, 0x49, 0x47, 0x50, 0x51, 0x2c, 0x47, 0x53, 0x56, 0x2a, 0x32, 0x34, 0x0d, 0x0a, 0xb5, 0x62, 0x06, 0x01, 0x03, 0x00, 0xf0, 0x03, 0x00, 0xfd, 0x15}; //->Disable_GPGSV
const static byte Disable_GPRMC[] PROGMEM = {0x24, 0x45, 0x49, 0x47, 0x50, 0x51, 0x2c, 0x52, 0x4d, 0x43, 0x2a, 0x33, 0x41, 0x0d, 0x0a, 0xb5, 0x62, 0x06, 0x01, 0x03, 0x00, 0xf0, 0x04, 0x00, 0xfe, 0x17}; //->Disable_GPRMC
const static byte Disable_GPVTG[] PROGMEM = {0x24, 0x45, 0x49, 0x47, 0x50, 0x51, 0x2c, 0x56, 0x54, 0x47, 0x2a, 0x32, 0x33, 0x0d, 0x0a, 0xb5, 0x62, 0x06, 0x01, 0x03, 0x00, 0xf0, 0x05, 0x00, 0xff, 0x19}; //->Disable_GPVTG
const static byte Disable_GPZDA[] PROGMEM = {0x24, 0x45, 0x49, 0x47, 0x50, 0x51, 0x2c, 0x5a, 0x44, 0x41, 0x2a, 0x33, 0x39, 0x0d, 0x0a, 0xb5, 0x62, 0x06, 0x01, 0x03, 0x00, 0xf0, 0x08, 0x00, 0x02, 0x1f}; //->Disable_GPZDA


///////////////////////////////////////////////////////
// Jour, Heure, Minute depuis début année            //
///////////////////////////////////////////////////////

int days[] = {0,  0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334}; // Nombre de jours au début du mois dans une année non bissextile.
long DofY(int yr, int mo, int dy, int h, int mi)
{
  int DfY;
  long Min2Date;
  if (mo == 1 || mo == 2)
  {
    DfY = days[mo] + dy - 1;                 // pour tout type d'année, il calcule le nombre de jours pour janvier ou février
  }
  else if ((yr % 4 == 0 && yr % 100 != 0) ||  yr % 400 == 0)
  {
    DfY = days[mo] + dy + 1 - 1;
  }
  else
  {
    DfY = days[mo] + dy - 1;
  }
  Min2Date = (long)DfY * 1440l + (long)h * 60l + (long)mi;
  return (Min2Date);
}


///////////////////////////////////////////////////////
//                      SETUP                        //
///////////////////////////////////////////////////////
void setup()
{
  analogReference(DEFAULT); // Ref.interne du ADC
  gpsSerial.begin(9600);  // Demarage protocole de comm série du GPS
  pinMode(sw_sel, INPUT_PULLUP);// Bouton "Validation" en entrée, tiré à VCC au repos 
  pinMode(sw_up, INPUT_PULLUP);// Bouton "Incrémentation" en entrée, tiré à VCC au repos BL
  pinMode(sw_dwn, INPUT_PULLUP);// Bouton "Décrémentationn" en entrée, tiré à VCC au repos BL
  lcd.begin(16, 2); // Afficheur 2 lignes 16 caractères 

  ///////////////////////////////////////////////////////
  //             Variables en   EEPROM                 //
  ///////////////////////////////////////////////////////
  hour = EEPROM.read(0); //lire le décalage par rapport à gmt
  EEPROM.get(10, wp_lat); // Mémoire de la Latitude enregistrée
  EEPROM.get(15, wp_lon);  // Mémoire de la Longitude enregistrée
  EEPROM.get(25, Odometre); // Mémoire de la Distance parcourue enregistrée
  EEPROM.get(30, time_On);  // Mémoire du temps enregistré depuis le dernier reset
  if (isnan(Odometre))               //Conserve la valeur GMT enregistrée
  {
    Odometre = 0;
  }
  if (hour > 127)                       
  {
    gmt =  hour - 256;
  }
  else
  {
    gmt = hour;
  }
  ///////////////////////////////////////////////////////
  ///               R E V                             ///
  ///////////////////////////////////////////////////////
  lcd.clear();
  lcd.print(F(" amazing-vacuum"));
  lcd.setCursor(0, 1);
  lcd.print(F("  tubes.com"));  // Entete
  delay (2500);
  lcd.clear();
  lcd.print("GMT="); lcd.print(gmt); // Affiche le décalage GMT et la tension de la batterie
  float vbatt = (float) analogRead(batt) / vref + 0.3;
  lcd.setCursor(0, 1);
  lcd.print(F("Batterie:"));
  lcd.print(vbatt);
  lcd.print("V");
  delay (3000);
  lcd.clear();
  lcd.print(F("Distance:"));// Affiche la distance depuis le dernier reset 
  lcd.print(Odometre);
  delay (3000);
  lcd.clear();
  lcd.print("M-Lat: ");// Affiche la latitude et longitude du point mémorisé 
  lcd.print(wp_lat);
  lcd.setCursor(0, 1);
  lcd.print("M-Lon: ");
  lcd.print(wp_lon);
  delay (3000);
  lcd.clear();
  lcd.print("Tmp-Course:"); // Affiche la durée de la course depuis la dernière RaZ 
  lcd.setCursor(0, 1);
  lcd.print(time_On / 60000); lcd.print("Min");
  delay (3000);


  ///////////////////////////////////////////////////////
  //                    GPS Reset                      //
  ///////////////////////////////////////////////////////
  lcd.clear();

  if (gps.encode(gpsSerial.read()))
  {
    lcd.print("GPS RaZ");
    gpsSerial.write(gpsHot, sizeof(gpsHot));  gpsSerial.flush();
    gpsSerial.write( gpsDefault, sizeof( gpsDefault));  gpsSerial.flush();
    gpsSerial.write( Disable_GPDTM, sizeof( Disable_GPDTM));  gpsSerial.flush();
    gpsSerial.write( Disable_GPGBS, sizeof( Disable_GPGBS));  gpsSerial.flush();
    gpsSerial.write( Disable_GPGLL, sizeof( Disable_GPGLL));  gpsSerial.flush();
    gpsSerial.write( Disable_GPGRS, sizeof( Disable_GPGRS));  gpsSerial.flush();
    gpsSerial.write( Disable_GPGSA, sizeof( Disable_GPGSA));  gpsSerial.flush();
    gpsSerial.write( Disable_GPGSV, sizeof( Disable_GPGSV));  gpsSerial.flush();
    gpsSerial.write( Disable_GPVTG, sizeof( Disable_GPVTG));  gpsSerial.flush();
    gpsSerial.write( Disable_GPGGA, sizeof( Disable_GPGGA));  gpsSerial.flush();
  }
  lcd.setCursor(0, 1); lcd.println(F("Attente Signal  "));

}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
///                                    L O O P                                                             ///
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
void loop()
{
  while (gpsSerial.available())                       // Contrôle des données GPS
  {
    if (gps.encode(gpsSerial.read()))                 // Lecture des données codées
    {
      gps.get_position(&lat, &lon);                   // Extraction Latitude et Longitude
      gps.crack_datetime(&year, &month, &day, &hour, &minute, &second);
      lat = lat / 100;  //Supprime les dixième de secondes
      lon = lon / 100;
      hour = hour + gmt;
      if (hour > 128)  // En reference a gmt, ajustement du jour 
      {
        hour = hour + 24;
        day = day - 1;
      }
      
      Year_Min = DofY ((int)year, (int)month, (int)day, (int)hour, (int)minute);


      if (digitalRead(sw_sel) == LOW) {        //Le bouton est-il pressé ?
        lcd.clear();
        while (digitalRead(sw_sel) == HIGH) {} // Attente du relachement
        menu(0);    // Et saut à menu zéro... 
      }

      float speedf = gps.speed() * .01852;   // 1.852 /100  Conversion en Kmh

      if (speedf < .5)
      {
        speedf = 0;
      }
      ///////////////////////////////////////////////////////
      /// Mesure de distance ou ODOMETRE                  ///
      ///////////////////////////////////////////////////////
      if (cum_n >= 10)
      {
        if (lat2 == 0)
        {
          lat2 = lat;
          lon2 = lon;
        }
        cum_n = 0;
        float lon2_dec = lon2 / 10000.0;
        //convertir les décimales de degré en radian
        float lat1_Rad = lat / 10000.0 * rad2deg;
        float lon1_Rad = lon / 10000.0 * rad2deg;
        float lat2_Rad = lat2 / 10000.0 * rad2deg;
        float lon2_Rad = lon2 / 10000.0 * rad2deg;

        //Calcul de la distance en Km
        float latSin = sin((lat1_Rad - lat2_Rad) / 2);
        float lonSin = sin((lon1_Rad - lon2_Rad) / 2);

        float P2P = 2 * asin(sqrt((latSin * latSin) + cos(lat1_Rad) * cos(lat2_Rad) * (lonSin * lonSin)));
        P2P = P2P * 6371;                      //Rayon de la terre = 6371 km

        if (P2P < 5 && speedf != 0) 
        {
          Odometre += P2P;
          lat2 = lat;
          lon2 = lon;
          EEPROM.put(25, Odometre);  // Ecrit en EEprom la distance
          time_run = time_run + (millis() - time_old);
          
          EEPROM.put(30, millis()); // Ecrit en EEprom la durée
        }
        time_old = millis();
      }
      else
      {
        cum_n ++;
      }
      ///////////////////////////////////////////////////////
      //                 V I T E S S E                     //
      ///////////////////////////////////////////////////////

      dtostrf (speedf, 3, 1, msg) ;

      {
        lcd.setCursor(0, 0);
        lcd.print(msg); lcd.print("Km/h  ");
        Print_Azimut(8, 0);
        horloge(0, 1) ;
        ////////////////////////////////////////////////////////////////////////////
        //  Affichage alternatif de la distance,du temps & position enregistrée   //
        ////////////////////////////////////////////////////////////////////////////
        fonction++;
        switch (fonction / 3)
        {
          case 1:
            {
              Print_wpazimut(6, 1);
              EEPROM.put(30, millis());
            }
            break;

          case 2:
            {
              lcd.setCursor(6, 1);
              if (Odometre < 100)
              {
                dtostrf (Odometre, 1, 1, msg) ;
              }
              else
              {
                dtostrf (Odometre, 1, 0, msg) ;
              }
              lcd.print ("Dist:"); lcd.print (msg); lcd.print ("Km   ");
            }
            break;
          case 3:
            {
              lcd.setCursor(6, 1);
              lcd.print ("Tmp:");
              if (time_run > 10000000L)//msec
              {
                lcd.print (time_run / 3600000L);//msec
                lcd.print ("Hr   ");
              }
              else
              { 
                lcd.print (time_run / 60000L);//msec
                lcd.print ("Min   ");
              }
            }
            break;
          default:
            {
              fonction = 2;
            }
            break;
        }


      }
    }
  }
}

///////////////////////////////////////////////////////
//                     MENU                          //
///////////////////////////////////////////////////////
static char* menu_items [] = {"Select- +/-", "Lat-Long & Memo",
                                "GMT Rgl  ", "Distance"};
                             
void menu(int menu_index)
{
  lcd.clear();  lcd.print(menu_items[menu_index]);
  while (digitalRead(sw_sel) == LOW) {}; delay(5); //Appui du bouton
  while (digitalRead(sw_sel) != LOW)// Selecton à la remonté du bouton
  {
    if (digitalRead(sw_up) == LOW)
    {
      lcd.clear();
      while (digitalRead(sw_up) == LOW) {}; delay(200);    //Avance du menu
      menu_index ++;
      if (menu_index > 5)
      {
        menu_index = 0;
      }
      lcd.clear();
      lcd.setCursor(0, 1);
      lcd.print(menu_items[menu_index]);
    }
    if (digitalRead(sw_dwn) == LOW)
    {
      lcd.clear();
      while (digitalRead(sw_dwn) == LOW) {}; delay(200);   //Recul du menu
      menu_index -= 1;
      if (menu_index < 0)
      {
        menu_index = 5;
      }
      lcd.clear();
      lcd.print(menu_items[0]);
      lcd.setCursor(0, 1);
      lcd.print(menu_items[menu_index]);
    }
  }
  lcd.clear();
  while (digitalRead(sw_sel) == LOW) {}; delay(100);  //Validation du menu choisi..
  switch (menu_index)
  {
   
    case 1:
      ///////////////////////////////////////////////////////
      //      MEMORISATION DE LA LATITUDE ET LONGITUDE     //
      ///////////////////////////////////////////////////////
      {
        lcd.clear ();
        lcd.setCursor (0, 0);
        lcd.print("Tr-Ar pour Memo") ;
        lcd.setCursor (0, 1);
        lcd.print("Point Debut ") ;
        delay (1000); lcd.clear ();
        while (digitalRead(sw_sel) != LOW)
        {
          Print_Latitude (0, 0);
          Print_Longitude (0, 1);
          if (digitalRead(sw_dwn) == LOW)  //Mémorisation de l'endroit de référence en EEPROM
          {
            wp_lat  = lat;
            wp_lon  = lon;
            EEPROM.put(10, lat);
            EEPROM.put(15, lon);
            lcd.clear ();
            lcd.setCursor (0, 0);
            lcd.print("Point deb. SAUVE") ;
            delay (1500);
            break;
          }
          if (digitalRead(sw_up) == LOW)
          {
            lcd.clear ();
            lcd.setCursor (0, 0);
            lcd.print("Tr-Ar pour Memo") ;
            while (digitalRead(sw_up) == LOW) {};
            delay(1000);
            lcd.clear ();
          }
          gps.get_position(&lat, &lon);
          lat = lat / 100; //enlever les dizièmes de secondes
          lon = lon / 100;
        }
        lcd.clear ();
        while (digitalRead(sw_sel) == LOW) {};
      }
      break;

 
    case 2:
      ///////////////////////////////////////////////////////
      //                     GMT                           //
      ///////////////////////////////////////////////////////
      {
        while (digitalRead(sw_sel) != LOW)
        {
          if (digitalRead(sw_up) == LOW)
          {
            lcd.clear ();
            while (digitalRead(sw_up) == LOW) {} ;delay(200);
            gmt ++;
          }

          lcd.clear();
          lcd.print ("GMT");
          lcd.print(gmt);
          delay (300);

          if (gmt > 11) {
           gmt = -11; 
          }
        }
        lcd.clear ();
        while (digitalRead(sw_sel) == LOW) {} ;
        lcd.setCursor (0, 1);
        lcd.print ("Rgl GMT");
        lcd.print(gmt);
        EEPROM.write(0, gmt);
        delay(1000);
        return;
      }
      break;


      
    case 3:
      ///////////////////////////////////////////////////////
      //                     DISTANCE                      //
      ///////////////////////////////////////////////////////
      {
        Odometre = 0;
        time_run = 0;
        lat2 = lat;
        lon2 = lon;
        lcd.clear();  lcd.print("Distance RaZ");
        delay (1500); lcd.clear ();
      }
      break;
    default:
      {
      }
      return;
  }

}

///////////////////////////////////////////////////////
//                  HORLOGE                          //
///////////////////////////////////////////////////////
void horloge(int C, int R)
{
  lcd.setCursor(C, R);
  lcd.print(hour); lcd.print(":");
  if (minute < 10)
  {
    lcd.print("0");
  }
  lcd.print(minute);
  lcd.print(" ");
}
///////////////////////////////////////////////////////
//                    LATITUDE                       //
///////////////////////////////////////////////////////
void Print_Latitude(int C, int R)
{
  long latdeg = (lat / 10000);
  lcd.setCursor(C, R);
  lcd.print("Lat:");
  lcd.print(abs(latdeg));
  lcd.print((char)223);
  float x = (float)(lat - latdeg * 10000) * .6 / 100;
  lcd.print(x); lcd.print("'");
  if (lat > 0)
  {
    lcd.print ("N");
  }
  else
  {
    lcd.print("S");
  }
}
///////////////////////////////////////////////////////
//                    LONGITUDE                      //
///////////////////////////////////////////////////////
void Print_Longitude(int C, int R)
{
  long londeg = (lon / 10000);
  lcd.setCursor(C, R);
  lcd.print("Lon:");
  lcd.print(abs(londeg));
  lcd.print((char)223);
  float x = (float) (abs(lon) - abs(londeg) * 10000) * .6 / 100;
  lcd.print(x); lcd.print("'");
  if (lon > 0) {
    lcd.print("E");
  }
  else {
    lcd.print("W");
  }
}

///////////////////////////////////////////////////////
//                  AZIMUT Boussole                  //
///////////////////////////////////////////////////////
void Print_Azimut(int C, int R)
{
  int azimut = gps.course() / 100;
  lcd.setCursor(C, R);
  lcd.print(azimut);
  lcd.print((char)223);
  azimut = (int)((azimut + 22.5f) / 45);
  lcd.print (azimuts[azimut % 8]); lcd.print ("   ");
}

///////////////////////////////////////////////////////
//        DISTANCE et AZIMUT Du POINT MEMORISE       //
///////////////////////////////////////////////////////
void Print_wpazimut(int C, int R)
{

  float lat_Rad = lat / 10000.0 * rad2deg;//  Converti les décimales de degree en radian
  float lon_Rad = lon / 10000.0 * rad2deg;
  float wp_lat_Rad = wp_lat / 10000.0 * rad2deg;
  float wp_lon_Rad = wp_lon / 10000.0 * rad2deg;
  float latSin = sin((lat_Rad - wp_lat_Rad) / 2);//  Calcul la distance en KM
  float lonSin = sin((lon_Rad - wp_lon_Rad) / 2);
  float distancekm = 2 * asin(sqrt((latSin * latSin) + cos(lat_Rad) * cos(wp_lat_Rad) * (lonSin * lonSin)));
  distancekm *= 6371;                      // Rayon de la terre = 6,371 km
  float yy = sin(wp_lon_Rad - lon_Rad) * cos(wp_lat_Rad);
  float xx = cos(lat_Rad) * sin(wp_lat_Rad) - sin(lat_Rad) * cos(wp_lat_Rad) * cos(wp_lon_Rad - lon_Rad);
  float fwd_azimut = atan2(yy, xx);
  fwd_azimut = round(fwd_azimut * 57.2958);
  if (fwd_azimut < 0 )
  {
    fwd_azimut =  360 + fwd_azimut;
  }
  if (distancekm < 10)
  {
    dtostrf (distancekm, 2, 2, msg) ;
  }
  else
  {
    dtostrf (distancekm, 2, 0, msg) ;
  }
  lcd.setCursor(C, R); lcd.print (msg); lcd.print ("km");
  dtostrf (fwd_azimut, 2, 0, msg) ;
  lcd.print (msg); lcd.print((char)223); lcd.print ("  ");
}




